import cr from './core';

class Cookie {
  protected pluses: RegExp;
  protected defaults: object;
  protected raw: boolean;
  public json: boolean;
  public cookie: any;

  constructor() {
    this.pluses = /\+/g;
    this.defaults = {};
    this.raw = false;
    this.json = false;
    this.cookie = this.config;
  }

  encode(s: any) {
    return this.raw ? s : encodeURIComponent(s);
  }

  decode(s: any) {
    return this.raw ? s : decodeURIComponent(s);
  }

  stringifyCookieValue(value: any) {
    return this.encode(this.json ? JSON.stringify(value) : String(value));
  }

  parseCookieValue(s: any) {
    if (s.indexOf('"') === 0) {
      // This is a quoted cookie as according to RFC2068, unescape...
      s = s.slice(1, -1).replace(/\\"/g, '"').replace(/\\\\/g, '\\');
    }

    try {
      // Replace server-side written pluses with spaces.
      // If we can't decode the cookie, ignore it, it's unusable.
      // If we can't parse the cookie, ignore it, it's unusable.
      s = decodeURIComponent(s.replace(this.pluses, ' '));
      return this.json ? JSON.parse(s) : s;
    } catch (e) {
      return false;
    }
  }

  read(s: any, converter?: any) {
    let value = this.raw ? s : this.parseCookieValue(s);
    return cr.isFunction(converter) ? converter(value) : value;
  }

  config(key: any, value: any, options?: any) {
    if (value !== undefined && !cr.isFunction(value)) {
      // @ts-ignore
      options = cr.extend({}, this.defaults, options);

      if (typeof options.expires === 'number') {
        let days = options.expires, t = options.expires = new Date();
        t.setTime(+t + days * 864e+5);
      }

      return (document.cookie = [
        this.encode(key),
        '=',
        this.stringifyCookieValue(value),
        // use expires attribute, max-age is not supported by IE
        options.expires ? '; expires=' + options.expires.toUTCString() : '',
        options.path ? '; path=' + options.path : '',
        options.domain ? '; domain=' + options.domain : '',
        options.secure ? '; secure=' +  options.secure : '',
        options.SameSite ? '; SameSite=' +  options.SameSite : ''
      ].join(''));
    }

    // Read
    let result: any = key ? undefined : {};

    // To prevent the for loop in the first place assign an empty array
    // in case there are no cookies at all. Also prevents odd result when
    // calling $.cookie().
    let cookies = document.cookie ? document.cookie.split('; ') : [];

    for (let i = 0, l = cookies.length; i < l; i++) {
      let parts = cookies[i].split('=');
      let name = this.decode(parts.shift());
      let cookie = parts.join('=');

      if (key && key === name) {
        // If second argument (value) is a function it's a converter...
        result = this.read(cookie, value);
        break;
      }

      // Prevent storing a cookie that we couldn't decode.
      if (!key && (cookie = this.read(cookie)) !== undefined) {
        result[name] = cookie;
      }
    }

    return result;
  }

  removeCookie(key: any, options?: any) {
    if (this.cookie(key) === undefined) {
      return false;
    }

    // Must not alter options, thus extending a fresh object...
    // @ts-ignore
    this.cookie(key, '', cr.extend({}, options, {expires: -1}));
    return !this.cookie(key);
  }
}

export default new Cookie();
